package com.emato.cuspay.service.wx.imp;

import com.emato.cuspay.common.contant.Dict;
import com.emato.cuspay.dao.data.MerchPayCfg;
import com.emato.cuspay.dao.data.wx.WxCbPayDoc;
import com.emato.cuspay.dao.data.wx.WxPayError;
import com.emato.cuspay.dao.mapper.MerchCusCfgMapper;
import com.emato.cuspay.dao.mapper.MerchPayCfgMapper;
import com.emato.cuspay.dao.mapper.wx.WxCbPayDocMapper;
import com.emato.cuspay.dao.mapper.wx.WxPayErrorMapper;
import com.emato.cuspay.service.wx.CusPayService;
import com.emato.cuspay.support.msg.resp.ResponseMessage;
import com.emato.cuspay.support.msg.resp.ResponseStatus;
import com.emato.cuspay.common.core.db.IdWorkerAide;
import com.emato.cuspay.common.contant.TablePrimaryKeyPrefix;
import com.emato.cuspay.dto.WxResponseMsg;
import com.emato.cuspay.util.XmlUtils;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.http.HttpEntity;
import org.apache.http.NameValuePair;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.xml.bind.JAXB;
import java.io.IOException;
import java.io.StringReader;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @author zengjun
 * @version 1.0
 * 2018-05-15 10:18
 */

@Service
public class CustomServiceImp implements CusPayService {
    private static final Logger logger = LoggerFactory.getLogger(CusPayService.class);

    private final static String CHARSET = "utf-8";

    private final static int HTTP_OK = 200;

    private final static String FAIL = "FAIL";

    private final static String RETURN_CODE_FAIL = "RETURN_CODE_FAIL";

    private final static String SUCCESS = "SUCCESS";


    private SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    @Value("${wx.payment.declare.url}")
    private String declareURL;

    @Value("${wx.payment.query.url}")
    private String queryURL;

    @Autowired
    private WxCbPayDocMapper wxCbPayDocMapper;

    @Autowired
    private WxPayErrorMapper wxPayErrorMapper;

    @Autowired
    private MerchCusCfgMapper merchCusCfgMapper;

    @Autowired
    private MerchPayCfgMapper merchPayCfgMapper;

    public void declarePayment() {
        //获取待申报的支付单
        List<WxCbPayDoc> wxCbPayDocs = wxCbPayDocMapper.selectBeDeclared();
        if (wxCbPayDocs == null || wxCbPayDocs.isEmpty()) {
            logger.info("没有待申报微信推海关的支付数据");
            return;
        }

        //记录异常支付单订单
        List<WxPayError> errors = Lists.newArrayList();

        wxCbPayDocs.forEach(wxCbPayDoc -> {

            MerchPayCfg merchPayCfg = merchPayCfgMapper.selectByWxMercherSn(wxCbPayDoc.getMerchSn());
            if (merchPayCfg == null ) {
                logger.error("商户编号位["+wxCbPayDoc.getMerchSn()+"]的商户不存在");
                return;
            }
            if (merchPayCfg.getApiKey() == null) {
                logger.error("商户编号为["+wxCbPayDoc.getMerchSn()+"]的api key 为空");
                return;
            }

            //生成报关所需要的xml
            String xml = XmlUtils.map2Xml(assemblyDeclareXml(wxCbPayDoc, merchPayCfg.getApiKey()));
            logger.info("支付单申报数据xml:"+xml);

            CloseableHttpClient httpClient = HttpClients.createDefault();
            HttpPost httpPost = new HttpPost(declareURL);
            httpPost.addHeader("Content-Type", "text/html;charset=UTF-8");
            StringEntity strEntity = new StringEntity(xml, "utf-8");
            httpPost.setEntity(strEntity);
            CloseableHttpResponse  response  = null;
            try {
                response = httpClient.execute(httpPost);
                int status = response.getStatusLine().getStatusCode();
                if ( HTTP_OK == status) {
                    HttpEntity httpEntity = response.getEntity();
                    String strHttpEntity = EntityUtils.toString(httpEntity);
                    logger.info("Http Entity string:"+strHttpEntity);
                    //将xml 转换为java对象
                    StringReader reader = new StringReader(strHttpEntity);
                    WxResponseMsg wxResponseMsg = JAXB.unmarshal(reader, WxResponseMsg.class);

                    //设置返回状态及返回消息
                    wxCbPayDoc.setReturnCode(wxResponseMsg.getReturnCode());
                    wxCbPayDoc.setReturnMsg(wxResponseMsg.getReturnMsg());
                    //返回状态码为失败
                    if (RETURN_CODE_FAIL.equals(wxResponseMsg.getReturnCode()) ||
                                FAIL.equals(wxResponseMsg.getReturnCode())) {
                        //设置申报状态为失败
                        wxCbPayDoc.setDocStatus(Dict.PaymentDocStatus.i_03.getItem());
                        //设置通知商户
                        wxCbPayDoc.setNotifyMerch(Dict.Whether.Yes.getItem());
                        wxCbPayDocMapper.updateByPrimaryKeySelective(wxCbPayDoc);
                    }

                    //返回这状态码为成功 业务结果成功
                    if (SUCCESS.equals(wxResponseMsg.getReturnCode()) && SUCCESS.equals(wxResponseMsg.getResultCode())) {
                        wxCbPayDoc.setResultCode(wxResponseMsg.getResultCode());

                        //返回状态未申报
                        if (Dict.ResponseMsgState.UNDECLARED.getItem().equals(wxResponseMsg.getState())) {
                            wxCbPayDoc.setDocStatus(Dict.PaymentDocStatus.i_01.getItem());
                            wxCbPayDoc.setNotifyMerch(Dict.Whether.Yes.getItem());
                            wxCbPayDocMapper.updateByPrimaryKeySelective(wxCbPayDoc);
                        }
                        //申报中
                        if (Dict.ResponseMsgState.PROCESSING.getItem().equals(wxResponseMsg.getState())) {
                            wxCbPayDoc.setDocStatus(Dict.PaymentDocStatus.i_22.getItem());
                            wxCbPayDocMapper.updateByPrimaryKeySelective(wxCbPayDoc);
                        }
                        //申报已提交
                        if (Dict.ResponseMsgState.SUBMITTED.getItem().equals(wxResponseMsg.getState())) {
                            wxCbPayDoc.setDocStatus(Dict.PaymentDocStatus.i_10.getItem());
                            wxCbPayDoc.setNotifyMerch(Dict.Whether.Yes.getItem());
                            wxCbPayDocMapper.updateByPrimaryKeySelective(wxCbPayDoc);
                        }
                        //申报成功
                        if (Dict.ResponseMsgState.SUCCESS.getItem().equals(wxResponseMsg.getState())) {
                            wxCbPayDoc.setDocStatus(Dict.PaymentDocStatus.i_11.getItem());
                            wxCbPayDoc.setNotifyMerch(Dict.Whether.Yes.getItem());
                            wxCbPayDocMapper.updateByPrimaryKeySelective(wxCbPayDoc);
                        }
                        //申报失败
                        if (Dict.ResponseMsgState.FAIL.getItem().equals(wxResponseMsg.getState())) {
                            wxCbPayDoc.setDocStatus(Dict.PaymentDocStatus.i_03.getItem());
                            wxCbPayDoc.setNotifyMerch(Dict.Whether.Yes.getItem());
                            wxCbPayDocMapper.updateByPrimaryKeySelective(wxCbPayDoc);
                        }
                        //海关接口异常
                        if (Dict.ResponseMsgState.EXCEPT.getItem().equals(wxResponseMsg.getState())) {
                            wxCbPayDoc.setDocStatus(Dict.PaymentDocStatus.i_03.getItem());
                            wxCbPayDoc.setNotifyMerch(Dict.Whether.Yes.getItem());
                            wxCbPayDocMapper.updateByPrimaryKeySelective(wxCbPayDoc);
                        }
                    }

                    //返回状态码为成功 业务结果失败
                    if (SUCCESS.equals(wxResponseMsg.getReturnCode()) && FAIL.equals(wxResponseMsg.getResultCode())) {
                        wxCbPayDoc.setResultCode(wxResponseMsg.getResultCode());
                        wxCbPayDoc.setErrCode(wxResponseMsg.getErrCode());
                        wxCbPayDoc.setErrCodeDes(wxResponseMsg.getErrCodeDes());
                        wxCbPayDoc.setDocStatus(Dict.PaymentDocStatus.i_12.getItem());
                        //业务失败 通知商户
                        wxCbPayDoc.setNotifyMerch(Dict.Whether.Yes.getItem());
                        wxCbPayDocMapper.updateByPrimaryKeySelective(wxCbPayDoc);
                    }
                } else {
                    throw new IOException("request url:["+declareURL+"] response code:"+ status);
                }
            } catch (IOException e) {
                WxPayError wxPayError = assemblyWxPayError(wxCbPayDoc);
                wxPayError.setErrCode("-1");
                wxPayError.setErrMsg("支付推单调用海关推单接口["+declareURL+"]异常");
                errors.add(wxPayError);
            } finally {
                if (response != null) {
                    try {
                        response.close();
                    } catch (Exception ex) {
                        //eat
                    }
                }
            }

            if (!errors.isEmpty()) {
                logger.info("支付单报关异常数据集合errors:[]"+errors);

                int result = wxPayErrorMapper.insertWxPayErrorBatch(errors);
                if (result <= 0) {
                    logger.error("添加异常记录信息失败");
                }
            }

        });

    }

    /**
     * 组装报关所需要的报关数据
     * @param wxCbPayDoc 微信支付数据
     * @param key 微信Api密钥
     * @return
     */
    protected Map<String, String> assemblyDeclareXml(WxCbPayDoc wxCbPayDoc, String key) {
        //1.选择报关数据,排序
        SortedMap<String, String> sorted = Maps.newTreeMap();
        sorted.put("appid", wxCbPayDoc.getAppid());
        sorted.put("mch_id", wxCbPayDoc.getMchId());
        sorted.put("out_trade_no", wxCbPayDoc.getOutTradeNo());
        sorted.put("transaction_id", wxCbPayDoc.getTransactionId());
        sorted.put("customs", wxCbPayDoc.getCustoms());
        sorted.put("mch_customs_no", wxCbPayDoc.getMchCustomsNo());
        if (wxCbPayDoc.getDuty() != null) {
            sorted.put("duty", String.valueOf(wxCbPayDoc.getDuty()));
        }
        if (wxCbPayDoc.getActionType() != null) {
            sorted.put("action_type", wxCbPayDoc.getActionType());
        }
        if (wxCbPayDoc.getSubOrderNo() != null) {
            sorted.put("sub_order_no", wxCbPayDoc.getActionType());
        }
        if (wxCbPayDoc.getFeeType() != null) {
            sorted.put("fee_type", wxCbPayDoc.getFeeType());
        }
        if (wxCbPayDoc.getOrderFee() != null) {
            sorted.put("order_fee", String.valueOf(wxCbPayDoc.getOrderFee()));
        }
        if (wxCbPayDoc.getTransportFee() != null) {
            sorted.put("transport_fee", String.valueOf(wxCbPayDoc.getTransportFee()));
        }
        if (wxCbPayDoc.getProductFee() != null) {
            sorted.put("product_fee", String.valueOf(wxCbPayDoc.getProductFee()));
        }
        if (wxCbPayDoc.getCertType() != null ) {
            sorted.put("cert_type", wxCbPayDoc.getCertType());
        }
        if (wxCbPayDoc.getCertId() != null) {
            sorted.put("cert_id", wxCbPayDoc.getCertId());
        }
        if (wxCbPayDoc.getName() != null) {
            sorted.put("name", wxCbPayDoc.getName());
        }

        //2.生成签名
        StringBuilder sb = new StringBuilder();
        for (Map.Entry<String,String> entry : sorted.entrySet()) {
            sb.append(entry.getKey()).append("=").append(entry.getValue()).append("&");
        }
        sb.append("key").append("=").append(key);
        //签名MD5 加密
        String sign = DigestUtils.md5Hex(sb.toString()).toUpperCase();

        sorted.put("sign", sign);
        return sorted;
    }

    public ResponseMessage addWxCbPayDoc(WxCbPayDoc wxCbPayDoc) {
        wxCbPayDoc.setWxPaySn(TablePrimaryKeyPrefix.wx_cb_pay_type + IdWorkerAide.nextId());
        //设置支付单状态 待审核
        wxCbPayDoc.setDocStatus(Dict.PaymentDocStatus.i_00.getItem());

        int result = wxCbPayDocMapper.insertSelective(wxCbPayDoc);
        if (result > 0) {
            return new ResponseMessage.Builder()
                    .setCode(ResponseStatus.SUCCESS.code)
                    .setMsg("入库成功")
                    .build();
        }
        return  new ResponseMessage.Builder()
                .setCode(ResponseStatus.ERROR.code)
                .setMsg("入库失败")
                .build();
    }

    /**
     * 支付单状态改变 异步通知商户
     */
    public void notifyMecher() {
        //查询需要通知商户的支付单
        List<WxCbPayDoc> wxCbPayDocs = wxCbPayDocMapper.selectNotify();
        if (wxCbPayDocs == null || wxCbPayDocs.isEmpty()) {
            logger.info("没有要通知的商户的支付信息");
            return;
        }

        //记录异常通知
        List<WxPayError> errors = Lists.newArrayList();
        //记录不需要通知的商户的支付信息
        List<WxCbPayDoc> notifyOffs = Lists.newArrayList();

        wxCbPayDocs.forEach((WxCbPayDoc wxCbPayDoc) -> {
            //获取商户支付信息
            MerchPayCfg merchPayCfg = merchPayCfgMapper.selectByWxMercherSn(wxCbPayDoc.getMerchSn());
            if (merchPayCfg == null) {
                logger.error("商户编号为["+wxCbPayDoc.getMerchSn()+"]的商户配置信息不存在");
                //如果不存在商户信息 停止通知
                WxPayError wxPayError = assemblyWxPayError(wxCbPayDoc);
                wxPayError.setErrCode("-3");
                wxPayError.setErrMsg("商户编号为["+wxCbPayDoc.getMerchSn()+"]的商户配置信息不存在");
                errors.add(wxPayError);
                notifyOffs.add(wxCbPayDoc);
                return;
            }
            if (merchPayCfg.getNotifyUrl() == null) {
                logger.error("商户编号为["+wxCbPayDoc.getMerchSn()+"]的回调通知接口为空");
                // 如果回调接口为空 停止通知该商户
                WxPayError wxPayError = assemblyWxPayError(wxCbPayDoc);
                wxPayError.setErrCode("-4");
                wxPayError.setErrMsg("商户编号为["+wxCbPayDoc.getMerchSn()+"]的商户的回调通知为空");
                errors.add(wxPayError);
                notifyOffs.add(wxCbPayDoc);
                return;
            }
            String notify_str = merchPayCfg.getNotifyUrl();

            CloseableHttpClient httpClient = HttpClients.createDefault();
            HttpPost httpPost = new HttpPost(notify_str);
            CloseableHttpResponse response = null;
            try {
                //通知商户数据填充 json格式数据
                httpPost.addHeader("Content-Type", "application/x-www-form-urlencoded;charset=UTF-8");

                //设置参数
                List<NameValuePair> pairs = Lists.newArrayList();
                pairs.add(new BasicNameValuePair("sn","mgt111111"));
                pairs.add(new BasicNameValuePair("name","test"));
                UrlEncodedFormEntity formEntity = new UrlEncodedFormEntity(pairs, CHARSET);

                httpPost.setEntity(formEntity);

                response = httpClient.execute(httpPost);
                if (HTTP_OK == response.getStatusLine().getStatusCode()) {
                    logger.info("response message:"+ EntityUtils.toString(response.getEntity()));
                    //处理商户返回的状态
                    wxCbPayDoc.setNotifyMerch(Dict.Whether.No.getItem());
                    wxCbPayDocMapper.updateByPrimaryKeySelective(wxCbPayDoc);
                } else {
                    throw  new IOException("request url:["+notify_str+"] response code:\"+ status");
                }
            }catch (IOException e) {
                logger.error("http post ["+notify_str+"] exception:" + e.getMessage());
                WxPayError wxPayError = assemblyWxPayError(wxCbPayDoc);
                wxPayError.setErrCode("-2");
                wxPayError.setErrMsg("请求商户回掉接口["+notify_str+"]异常");
                errors.add(wxPayError);
                notifyOffs.add(wxCbPayDoc);
            } finally {
                if (response != null) {
                    try {
                        response.close();
                    } catch (Exception e) {
                        //eat
                    }
                }
            }

        });

        if (!errors.isEmpty()) {
            logger.info("商户回调通知异常数据集合errors:[]"+errors);

            int result = wxPayErrorMapper.insertWxPayErrorBatch(errors);
            if (result <= 0) {
                logger.error("添加异常记录信息失败");
            }
        }

        if (!notifyOffs.isEmpty()) {
            int result = wxCbPayDocMapper.insertBatch(notifyOffs);
            if (result <= 0) {
                logger.error("添加异常记录信息失败");
            }
        }
    }


    /**
     * 填充WxpayError
     * @param wxCbPayDoc 支付信息
     * @return
     */
    protected WxPayError assemblyWxPayError(WxCbPayDoc wxCbPayDoc) {
        WxPayError wxPayError = new WxPayError();
        wxPayError.setErrorSn(TablePrimaryKeyPrefix.wx_pay_error_type + IdWorkerAide.nextId());
        wxPayError.setWxPaySn(wxCbPayDoc.getWxPaySn());
        wxPayError.setAppid(wxCbPayDoc.getAppid());
        wxPayError.setMerchSn(wxCbPayDoc.getMerchSn());
        wxPayError.setMerchName(wxCbPayDoc.getMerchName());
        wxPayError.setThirdPartyMerchCode(wxCbPayDoc.getThirdPartyMerchCode());
        wxPayError.setThirdPartyMerchName(wxCbPayDoc.getThirdPartyMerchName());
        wxPayError.setAppid(wxCbPayDoc.getAppid());
        wxPayError.setMchId(wxCbPayDoc.getMchId());
        wxPayError.setOutTradeNo(wxCbPayDoc.getOutTradeNo());
        wxPayError.setTransactionId(wxCbPayDoc.getTransactionId());
        wxPayError.setSubOrderNo(wxCbPayDoc.getSubOrderNo());
        wxPayError.setCustoms(wxCbPayDoc.getCustoms());
        wxPayError.setCreateTime(sdf.format(new Date()));
        return wxPayError;
    }

    /**
     * 查询支付单状态，如果支付单状态改变同步数据
     */
    public void QueryPaymentStateAndSynDB() {

    }
}
